@@ -16,6 +16,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Btaurus.UCSC.bosTau4",
   "BSgenome.Btaurus.UCSC.bosTau6",
   "BSgenome.Btaurus.UCSC.bosTau8",
+  "BSgenome.Btaurus.UCSC.bosTau9",
   "BSgenome.Celegans.UCSC.ce2",
   "BSgenome.Celegans.UCSC.ce6",
   "BSgenome.Celegans.UCSC.ce10",
@@ -30,11 +31,14 @@ BSgenome_datapkgs <- c(
   "BSgenome.Drerio.UCSC.danRer6",
   "BSgenome.Drerio.UCSC.danRer7",
   "BSgenome.Drerio.UCSC.danRer10",
+  "BSgenome.Drerio.UCSC.danRer11",
+  "BSgenome.Dvirilis.Ensembl.dvircaf1",
   "BSgenome.Ecoli.NCBI.20080805",
   "BSgenome.Gaculeatus.UCSC.gasAcu1",
   "BSgenome.Ggallus.UCSC.galGal3",
   "BSgenome.Ggallus.UCSC.galGal4",
   "BSgenome.Ggallus.UCSC.galGal5",
+  "BSgenome.Ggallus.UCSC.galGal6",
   "BSgenome.Hsapiens.UCSC.hg17",
   "BSgenome.Hsapiens.UCSC.hg18",
   "BSgenome.Hsapiens.UCSC.hg19",
@@ -46,6 +50,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Mmulatta.UCSC.rheMac2",
   "BSgenome.Mmulatta.UCSC.rheMac3",
   "BSgenome.Mmulatta.UCSC.rheMac8",
+  "BSgenome.Mmulatta.UCSC.rheMac10",
   "BSgenome.Mmusculus.UCSC.mm8",
   "BSgenome.Mmusculus.UCSC.mm9",
   "BSgenome.Mmusculus.UCSC.mm10",
